#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#                   Selector.py V3 - (c) Berthold Hinz 2010
# ==============================================================================
#
#    RANGE SELECTION PLUGIN FOR GIMP
#
#    Needs at least Python 2.6.4!
#
# ==============================================================================
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# ==============================================================================

import pygtk
import gtk
import gtk.glade
import colorsys
import os
import gimp, gimpplugin
from gimpenums import *
import sys
pdb = gimp.pdb

def hex_to_rgb(value):
    value = value.lstrip('#')
    lv = len(value)
    return tuple(int(value[i:i+lv/3], 16) for i in range(0, lv, lv/3))

def hue(r,g,b):
    r=1.0/255*r
    g=1.0/255*g
    b=1.0/255*b
    max=r
    min=r
    if g>max: max=g
    if g<min: min=g
    if b>max: max=b
    if b<min: min=b
    if max==min:
        hue=0
    elif max==r:
        hue=60*(0+(g-b)/(max-min))
    elif max==g:
        hue=60*(2+(b-r)/(max-min))
    elif max==b:
        hue=60*(4+(r-g)/(max-min))
    if hue<0: hue=hue+360
    hue=(hue/360.0*255.0)
    return hue

class gui:

    def __init__(self): 
        gladefile = sys.path[0] + "/" + "selector.gui"
        self.wTree=gtk.glade.XML(gladefile) 
        dic = {"on_cancel": (gtk.main_quit),
                "on_combo_channel_changed": self.set_channel,
                "on_opt_highlights_toggled": self.set_range,
                "on_opt_midtones_toggled": self.set_range,
                "on_opt_shadows_toggled": self.set_range,
                "on_opt_custom_toggled": self.set_range,
                "on_opt_result_selection_toggled": self.set_result,
                "on_hscale_left_border_value_changed": self.set_left,
                "on_hscale_right_border_value_changed": self.set_right,
                "on_cmd_test_clicked": self.test,
                "on_cmd_apply_clicked": self.apply,
                "on_about": self.about,
                "on_hscale_ED_change_value": self.set_ed,
                "on_hscale_width_change_value": self.set_range,
                "on_hscale_feathering_change_value": self.set_feathering,
                "on_opt_mode_normal_toggled": self.set_mode,
                "on_opt_format_gray_toggled": self.set_format,
                "on_opt_format_bw_toggled": self.set_format,
                "on_cmd_colour_color_set": self.set_colour,
                "on_col_left_color_set": self.set_Lcolour,
                "on_col_right_color_set": self.set_Rcolour,
                "on_cmd_undo_clicked": self.reset,
                "on_cmd_quit_clicked": self.quit}

        self.wTree.signal_autoconnect(dic) 
        self.win = self.wTree.get_widget("window_main") 
        self.win.connect("delete_event", self.on_delete_event) 
        self.win.show() 
        dialog=self.win
        self.about = self.wTree.get_widget("aboutdialog1") 
        self.ch=self.wTree.get_widget("combo_channel")
        self.l_op=self.wTree.get_widget("lbl_operation")
        self.hl = self.wTree.get_widget("opt_highlights")
        self.mt = self.wTree.get_widget("opt_midtones")
        self.sh = self.wTree.get_widget("opt_shadows")
        self.cust = self.wTree.get_widget("opt_custom")
        self.hs_feather = self.wTree.get_widget("hscale_feathering")
        self.norm = self.wTree.get_widget("opt_mode_normal")
        self.inv = self.wTree.get_widget("opt_mode_inverted")
        self.sel = self.wTree.get_widget("opt_result_selection")
        self.mask = self.wTree.get_widget("opt_result_mask")
        self.gray = self.wTree.get_widget("opt_format_gray")
        self.bw = self.wTree.get_widget("opt_format_bw")
        self.ed = self.wTree.get_widget("hscale_ED")
        self.hs_width = self.wTree.get_widget("hscale_width")
        self.hs_left = self.wTree.get_widget("hscale_left_border")
        self.hs_right = self.wTree.get_widget("hscale_right_border")
        self.col = self.wTree.get_widget("cmd_colour")
        self.col_left = self.wTree.get_widget("col_left")
        self.col_right = self.wTree.get_widget("col_right")
        self.test = self.wTree.get_widget("cmd_test")
        self.undo = self.wTree.get_widget("cmd_undo")
        self.apply = self.wTree.get_widget("cmd_apply")
        self.quit = self.wTree.get_widget("cmd_quit")
        self.l_width = self.wTree.get_widget("lbl_width")
        self.l_left = self.wTree.get_widget("lbl_left_border")
        self.l_right = self.wTree.get_widget("lbl_right_border")
        self.l_format = self.wTree.get_widget("lbl_mask_format")
        self.l_ed = self.wTree.get_widget("lbl_ED")
        self.ch.set_active(0)
        self.hl.set_active(True)
        self.hs_width.set_sensitive(True)
        self.hs_left.set_sensitive(False)
        self.hs_right.set_sensitive(False)
        self.l_width.set_sensitive(True)
        self.l_left.set_sensitive(False)
        self.l_right.set_sensitive(False)
        self.gray.set_sensitive(False)
        self.bw.set_sensitive(False)
        self.l_format.set_sensitive(False)
        self.apply.set_sensitive(False)

        ac=0
        global bgc
        bgc=gimp.get_background()
        highlighting_color=(255,0,0)
        gimp.set_background(highlighting_color)
        self.initial_sel_exists=1-pdb.gimp_selection_is_empty(img)
        self.initial_selection=None
        if self.initial_sel_exists==1:
            drw=pdb.gimp_image_get_active_drawable(img)
            pdb.gimp_selection_save(img)
            self.initial_selection=pdb.gimp_image_get_active_channel(img)
            pdb.gimp_image_set_active_layer(img,drw)
        else:
            self.initial_selection=None
        self.mask_added=False

        if len(img.layers)==1:
            pdb.gimp_image_set_active_layer(img,img.layers[0])

    def on_delete_event(self, widget, event): 
        if img.layers[0].name<>"Selector preview":
##            self.win.set_sensitive(False)
##            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, None)
##            dialog.set_markup('<b>Are you sure you want to quit Selektor?</b>') 
##            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
##            answer = dialog.run()
##            if answer == -8: ## YES
                dialog.destroy()
                return False
##            if answer == -9: ## NO
##                dialog.destroy()
##                return True
        else:
            self.win.set_sensitive(False) 
            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, None)
            dialog.set_markup('You decided to quit while changes shown by the preview layer have not been applied.<b>\n\nDo you want to remove the preview layer and reset the selection?</b>\n\n(close this dialog if you do not want to quit)') 
            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
            dialog.set_title("Selector: Question") 
            answer = dialog.run()
            dialog.destroy()
            if answer == -8: ## YES = quit
                dialog.destroy()
                pdb.gimp_image_undo_group_start(img)
                pdb.gimp_image_remove_layer(img,img.layers[0])
                pdb.gimp_displays_flush()
                pdb.gimp_selection_none(img)
                pdb.gimp_selection_load(self.initial_selection)
                pdb.gimp_image_undo_group_end(img)
                self.win.destroy()
            elif answer == -9: ## NO = don't quit
                dialog.destroy()
            else:
                dialog.destroy()
                return True

    def set_range(self,widget, par3=None, pare4=None):
        self.changed()
        is_custom=self.cust.get_active()
        self.hs_width.set_sensitive(not(is_custom))
        self.hs_left.set_sensitive(is_custom)
        self.hs_right.set_sensitive(is_custom)
        self.l_width.set_sensitive(not(is_custom))
        self.l_left.set_sensitive(is_custom)
        self.l_right.set_sensitive(is_custom)
        width_of_range=self.hs_width.get_value()
        if  self.hl.get_active()==True:
            area="highlights"
            min=255-255.0/100*width_of_range
            max=255
            self.hs_left.set_value(min)
            self.hs_right.set_value(max)
        elif self.sh.get_active()==True:
            area="shadows"
            min=0
            max=255.0/100*width_of_range
            self.hs_left.set_value(min)
            self.hs_right.set_value(max)
        elif self.mt.get_active()==True:
            area="midtones"
            min=127.5-(255.0/100*width_of_range/2.0)
            max=127.5+(255.0/100*width_of_range/2.0)
            self.hs_left.set_value(min)
            self.hs_right.set_value(max)
        else:
            border_l=self.hs_left.get_value()
            border_r=self.hs_right.get_value()
            min=border_l
            max=border_r
            if max<min:
                min=border_r
                max=border_l
            widthrange=round(100*(max-min)/255.0)
            self.hs_width.set_value(widthrange)
        min=round(min)
        max=round(max)

    def set_channel(self,widget):
        self.changed()
        ac=self.ch.get_active()
        is_ed=(ac==9)
        if is_ed==True:
            self.ed.show()
            self.l_ed.show()
        else:
            self.ed.hide()
            self.l_ed.hide()

        is_hue=(ac==10)
        self.hl.set_sensitive(not(is_hue))
        self.mt.set_sensitive(not(is_hue))
        self.sh.set_sensitive(not(is_hue))
        if is_hue:
            self.cust.set_active(True)
        cb_visible=not(ac>1 and ac<10)
        if cb_visible==True:
            self.col_left.show()
            self.col_right.show()
            self.set_left(self)
            self.set_right(self)
        else:
            self.col_left.hide()
            self.col_right.hide()

    def set_format(self,widgets):
        is_mask=self.mask.get_active()
        self.gray.set_sensitive(is_mask)
        self.bw.set_sensitive(is_mask)
        self.l_format.set_sensitive(is_mask)
        self.changed()

    def set_result(self,widgets):
        is_mask=self.mask.get_active()
        self.gray.set_sensitive(is_mask)
        self.bw.set_sensitive(is_mask)
        self.l_format.set_sensitive(is_mask)

    def set_width(self,widgets):
        self.changed()
    
    def set_left(self,widgets):
        self.changed()
        r=self.hs_right.get_value()
        l=self.hs_left.get_value()
        if l>=r:
            l=r-1
        self.hs_left.set_value(l)
        self.set_range(self,widgets)
        c=int(l*257)
        self.col_left.set_color(gtk.gdk.Color(c,c,c))
        ac=self.ch.get_active()
        cb_sensitive=not(ac>1 and ac<10)
        if ac==10:
            h=l/256.0
            r,g,b = colorsys.hsv_to_rgb(h, 1.0, 1.0)
            r=int(r*255)*257
            g=int(g*255)*257
            b=int(b*255)*257
            self.col_left.set_color(gtk.gdk.Color(r,g,b))

    def set_right(self,widgets):
        self.changed()
        r=self.hs_right.get_value()
        l=self.hs_left.get_value()
        if r<=l:
            r=l+1
        self.hs_right.set_value(r)
        self.set_range(self,widgets)
        c=int(r*257)
        self.col_right.set_color(gtk.gdk.Color(c,c,c))
        ac=self.ch.get_active()
        cb_sensitive=not(ac>1 and ac<10)

        if ac==10:
            h=r/256.0
            r,g,b = colorsys.hsv_to_rgb(h, 1.0, 1.0)
            r=int(r*255)*257
            g=int(g*255)*257
            b=int(b*255)*257
            self.col_right.set_color(gtk.gdk.Color(r,g,b))

    def rgb(hue):
        intensity=100
        saturation=100
        H = PI * 2.0 * hue / 255.0
        I = sqrt(6) * intensity/255.0
        M1 = saturation / 255.0 * sin(H) / sqrt(6)
        M2 = saturation / 255.0 * cos(H) / sqrt(2)
        R = (2.0*M1 + I) / 4.0 * sqrt(6)
        G = (-M1 + M2 + I) / 4.0 * sqrt(6)
        B = (-M1 - M2 + I) / 4.0 * sqrt(6)
        Red = int(R * 255)
        Green = int(G * 255)
        Blue = int(B * 255)
        return Red, Green, Blue

    def set_feathering(self,widgets, par3=None, par4=None):
        feathering=int(self.hs_feather.get_value())
        self.changed()

    def set_mode(self,widgets):
        if img.layers[0].name== "Selector preview":
            pdb.plug_in_threshold_alpha(img,img.layers[0],255)
            pdb.gimp_selection_invert(img)
            h=self.col.get_color()
            if str(h)=="#f00":
                        r=255
                        g=0
                        b=0
            else:
                r,g,b=hex_to_rgb(str(h))
                if len(str(h))==4:
                    r=r*17
                    g=g*17
                    b=b*17
                elif len(str(h))>7:
                    r=int(r/256.0)
                    g=int(g/256.0)
                    b=int(b/256.0)
            gimp.set_background(r,g,b)
            pdb.gimp_edit_fill(img.layers[0],1)
            gimp.set_background(bgc)
            img.layers[0].opacity=100.0
            pdb.gimp_displays_flush()

    def set_format(self,widgets):
        pass

    def test(self,widget): 
        pdb.gimp_image_undo_group_start(img)

        # falls bereits preview vorhanden: entfernen
        if img.layers[0].name=="Selector preview":
            pdb.gimp_image_remove_layer(img,img.layers[0])
            pdb.gimp_selection_none(img)
        
        # Parameter einlesen
        min=self.hs_left.get_value()
        max=self.hs_right.get_value()
        feathering=int(self.hs_feather.get_value())
        amount=self.ed.get_value()

        ## RGB-Werte für Markierungsfarbe bestimmen und setzen
        h=self.col.get_color()
        if str(h)=="#f00": 
            r=255
            g=0
            b=0
        else:
            r,g,b=hex_to_rgb(str(h))
            r=int(r/256.0)
            g=int(g/256.0)
            b=int(b/256.0)
        highlighting_color=(r,g,b)
        gimp.set_background(highlighting_color)

        # Schaltflaechenstatus aktualisieren 
        self.changed()

        # aktuelle Zeichenflaeche ermitteln
        drw=pdb.gimp_image_get_active_drawable(img)
        
        # Fehlermeldung, falls Ebenenmaske aktiviert
        if pdb.gimp_drawable_is_layer_mask(drw)==1:
            self.win.set_sensitive(False) 
            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, None)
            dialog.set_markup('<b>Selector does not work on layer masks. \nPlease activate the layer and try again!</b>') 
            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
            dialog.set_title("Selector: Warning") 
            answer = dialog.run()
            dialog.destroy()
            return

        # aktuelle Ebene ermitteln
        if len(img.layers)==1:
            pdb.gimp_image_set_active_layer(img,img.layers[0])
        current_layer=pdb.gimp_image_get_active_layer(img)

        # Fehlermeldung, falls nicht oberste Ebene aktiviert
        if current_layer<>img.layers[0]:
            self.win.set_sensitive(False) 
            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, None)
            dialog.set_markup('Selector works on the top layer of the image, but currently the active layer is not the top one.<b>\n\nDo you want to create the preview for the top layer now?</b>') 
            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
            dialog.set_title("Selector: Question") 
            answer = dialog.run()
            dialog.destroy()
            if answer == -8:
                dialog.destroy()
                pdb.gimp_image_set_active_layer(img,img.layers[0])
            if answer == -9:
                dialog.destroy()
                return
        # Schaltflaechenstatus aktualisieren
        self.test.set_sensitive(False)
        self.apply.set_sensitive(True)

        # Kanal setzen
        channel=self.ch.get_active()
        # 0 Value
        # 1 Lightness
        # 2 Saturation
        # 3 Red
        # 4 Green
        # 5 Blue
        # 6 Cyan
        # 7 Magenta
        # 8 Yellow
        # 9 Edges
        # 10 Hue
        cl={0:'V',1:'L',2:'S',3:'S',4:'S',5:'S',6:'S',7:'S',8:'S', 9:'V', 10:'H'}
        channel_number={0:2,1:0, 2:1, 3:1,4:1,5:1,6:1,7:1,8:1, 9:2, 10:0}
        channel_letter=cl[channel]
        
        # zum Kanal gehoerendes Farbmodell bestimmen
        if "RGB".find(channel_letter)>-1:
            cmod="RGB"
        elif "HSV".find(channel_letter)>-1:
            cmod="HSV"
        elif "CMY".find(channel_letter)>-1:
            cmod="CMY"
        elif "LAB".find(channel_letter)>-1:
            cmod="LAB"
        
        # Kanal für einzelne Farbsaettigung erstellen, indem Sättigung aller Farben, bis auf die der betreffenden, auf 0 gesetzt werden
        if channel>2 and channel <10:
            drw=img.layers[0]
            layer_desat = drw.copy(True)
            layer_desat.mode = NORMAL_MODE
            layer_desat.opacity=100.0
            img.add_layer(layer_desat,-1)
            drw=layer_desat
            ## fuer alle ausser Rot: Rottoene entsättigen
            if channel<>3 and channel<10:
                pdb.gimp_hue_saturation(drw,1,0,0,-100)
            ## fuer alle ausser Gelb: Gelbtoene entsättigen
            if channel<>8 and channel<10:
                pdb.gimp_hue_saturation(drw,2,0,0,-100)
            ## fuer alle ausser Gruen: Gruentoene entsaettigen
            if channel<>4 and channel<10:
                pdb.gimp_hue_saturation(drw,3,0,0,-100)
            ## fuer alle ausser Cyan: Cyantoene entsättigen
            if channel<>6 and channel<10:
                pdb.gimp_hue_saturation(drw,4,0,0,-100)
            ## fuer alle ausser Blau: Blautoene entsättigen
            if channel<>5 and channel<10:
                pdb.gimp_hue_saturation(drw,5,0,0,-100)
            ## fuer alle ausser Magenta: Magentatoene entsättigen
            if channel<>7 and channel<10:
                pdb.gimp_hue_saturation(drw,6,0,0,-100)
            ## falls Edge Detection:
            if channel==9:
                pdb.plug_in_edge(img,drw,amount,1,0)

        # aktuelle Zeichenflaeche nach ermitteltem Farbmodell zerlegen
        drw=img.layers[0] # erforderlich, da bei wiederholtem Aufruf sonst u. U. drw=channel
        DEC_img=pdb.plug_in_decompose(img, drw, cmod, 1)[0]
        
        # leere Markierungsebene erstellenund hinzufuegen
        width = drw.width
        height = drw.height
        layer_check = gimp.Layer(img, "Selector preview", width, height, RGB_IMAGE, 100, NORMAL_MODE)
        layer_check.mode = NORMAL_MODE
        layer_check.opacity=0.0
        img.add_layer(layer_check, 0)
        
        # benoetigten Kanal aus zerlegtem Bild kopieren und mit Markierungsebene verankern
        pdb.gimp_edit_copy(DEC_img.layers[channel_number[channel]])
        pdb.gimp_edit_paste(layer_check,0)
        pdb.gimp_floating_sel_anchor(img.layers[0])

        # temporaer zerlegtes Bild wird nicht mehr benoetigt
        pdb.gimp_image_delete(DEC_img)

        # oberste Ebene aktiveren
        drw=img.layers[0]
        layer_number=0

        # Auswahl erstellen
        pdb.gimp_selection_none(img)
        THR=int(max-min)/2+1 # Schwelle
        AM=int(min+THR) # Mitte des Wertebereichs
        CS=(AM,AM,AM) # Farbe in der Mitte des Wertebereichs

        ## alle Kanalbereiche zwischen den Grenzen werden durch Rot ersetzt
        pdb.plug_in_exchange (img,img.layers[layer_number],AM,AM,AM,255,0,0,THR,THR,THR)

        ## alle Rottöne werden ausgewaehlt
        pdb.gimp_by_color_select(img.layers[layer_number],(255,0,0), THR,0, True, True, feathering, False)

        # Auswahl invertieren, falls gewuenscht
        if self.inv.get_active()==1:
            pdb.gimp_selection_invert(img)

        # nicht ausgewaehlte Bereiche entfernen
        ## Alpha-Kanal hinzufuegen
        pdb.gimp_layer_add_alpha(img.layers[layer_number])
        ## Auswahl invertieren
        pdb.gimp_selection_invert(img)
        ## ausgewaehte Bereiche durch transparente Pixel ersetzen
        pdb.plug_in_threshold_alpha(img,img.layers[layer_number],255)
        ## Auswahl zurueck invertieren
        pdb.gimp_selection_invert(img)
        
        # Auswahl mit Farbe fuellen
        pdb.gimp_edit_fill(img.layers[layer_number],1)
        gimp.set_background(bgc)
        drw=layer_check
        img.layers[0].opacity=100.0
        pdb.gimp_displays_flush()

        # falls ueberfluessige Ebene: loschen
        if channel>2 and channel<10:
            pdb.gimp_image_remove_layer(img,img.layers[1])

        # progress bar wird nicht zurueckgesetzt (Bug in Gimp?): hier manuell Wert und Text ueberschreiben
        pdb.gimp_displays_flush()
        pdb.gimp_progress_set_text(" ")
        pdb.gimp_progress_update(0.0)
        pdb.gimp_progress_cancel

        # Auswahl zwischenspeichern
        if len(img.channels)>0:
            for i in xrange(len(img.channels)):
                if img.channels[i].name=="Selector":
                    cha=img.channels[i]
                    img.remove_channel(cha)
        self.stored_selection=pdb.gimp_selection_save(img)
        self.stored_selection.name="Selector"
        pdb.gimp_image_set_active_layer(img,img.layers[0])

        pdb.gimp_image_undo_group_end(img)

    def apply(self,widget):
        pdb.gimp_image_undo_group_start(img)
        self.apply.set_sensitive(False)
        self.undo.set_sensitive(True)

        # Auswahl erstellen
        ## falls Auswahl erstellt werden soll
        if self.sel.get_active()==True:
            pdb.gimp_image_remove_layer(img,img.layers[0])
            pdb.gimp_displays_flush()

        # Ebenenmaske erstellen
        ## falls Ebenenmaske erstellt werden soll
        else:
            pdb.gimp_image_remove_layer(img,img.layers[0])
            drw=img.layers[0]
            layer_temp = drw.copy(True)
            layer_temp.mode = NORMAL_MODE
            layer_temp.opacity=100.0
            img.add_layer(layer_temp,-1)
            drw=layer_temp
            pdb.gimp_displays_flush()
            self.win.set_sensitive(False) 
##            pdb.gimp_image_undo_thaw(img)
##            # Sicherheitsabfrage, ob vorhandene Auswahlen geloescht werden sollen
##            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL, None)
##            dialog.set_markup('Selector is going to create a layer mask.<b>\n\nAll current selections will be removed!</b>') 
##            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
##            dialog.set_title("Selector: Message") 
##            answer = dialog.run()
##            dialog.destroy()
##            # Ablehnung
##            if answer <> -5:
##                self.apply.set_sensitive(True)
##                return

            ## neue Ebene erstellen
            drw=img.layers[1]
            layer_masked = drw.copy(True)
            layer_masked.mode = NORMAL_MODE
            layer_masked.name = "masked"
            layer_masked.opacity=100.0
            img.add_layer(layer_masked,-1)
            gimp.set_background(255,255,255)

            ## Maskenformat S/W
            if self.bw.get_active()==True:
                pdb.gimp_edit_fill(img.layers[0],1)
            pdb.gimp_edit_copy(layer_masked)
            
            ## loeschen, falls bereits Ebenenmaske vorhanden
            if pdb.gimp_layer_get_mask(img.layers[1]) <>None:
                pdb.gimp_layer_remove_mask(img.layers[1],1)
            
            ## Maske aus Previewebene erstellen und einfuegen
            mask = pdb.gimp_layer_create_mask(img.layers[0], ADD_BLACK_MASK)
            pdb.gimp_layer_add_mask(img.layers[2],mask)
            pdb.gimp_edit_paste(mask,0)
            self.mask_added=True
            drw=mask
            pdb.gimp_floating_sel_anchor(img.layers[0])
            
            ## falls Maskenformat S/W
            if self.bw.get_active()==True:
                pdb.gimp_levels_stretch(mask)

            # temporaere Maskenebene und Previewebene entfernen
            pdb.gimp_image_remove_layer(img,img.layers[0])
            pdb.gimp_image_remove_layer(img,img.layers[0])
            pdb.gimp_image_set_active_layer(img,img.layers[0])
            pdb.gimp_displays_flush()

        pdb.gimp_image_undo_group_end(img)
        self.win.destroy()

    def quit(self,widget):
        if img.layers[0].name=="Selector preview":
            self.win.set_sensitive(False) 
            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, None)
            dialog.set_markup('You decided to quit while changes shown by the preview layer have not been applied.<b>\n\nDo you want to remove the preview layer and reset the selection?</b>\n\n(close this dialog if you do not want to quit)') 
            dialog.connect("destroy", lambda w: self.win.set_sensitive(True))
            dialog.set_title("Selector: Question") 
            answer = dialog.run()
            dialog.destroy()
            if answer == -8: ## YES = quit
                dialog.destroy()
                pdb.gimp_image_undo_group_start(img)
                pdb.gimp_image_remove_layer(img,img.layers[0])
                pdb.gimp_selection_none(img)
                if self.initial_sel_exists==1:
                    pdb.gimp_selection_load(self.initial_selection)
                    pdb.gimp_image_remove_channel(img,self.initial_selection)
                pdb.gimp_image_undo_group_end(img)
                pdb.gimp_displays_flush()
                self.win.destroy()
            if answer == -9: ## NO = don't quit
                dialog.destroy()
        else:
            self.win.set_sensitive(False) 
            self.win.destroy()

    def reset(self,widget):
        self.test.set_sensitive(True)
        self.undo.set_sensitive(False)
        pdb.gimp_selection_none(img)
        if self.initial_sel_exists==1:
            pdb.gimp_selection_load(self.initial_selection)
        if self.mask_added==True and pdb.gimp_layer_get_mask(img.layers[0]) <>None:
                pdb.gimp_layer_remove_mask(img.layers[0],1)
        self.mask_added=False
        pdb.gimp_displays_flush()

    def about(self, widget):
        result = self.about.run()
        self.about.hide()

    def set_ed(self, widget,temp_a,temp_b):
        self.changed()

    def set_width(self, widget,temp_a,temp_b):
        self.changed()

    def set_operation(self, widget):
        self.changed()

    def set_colour(self, widget):
        h=self.col.get_color()
        r,g,b=hex_to_rgb(str(h))
        if len(str(h))==4:
            r=r*17
            g=g*17
            b=b*17
        elif len(str(h))>7:
            r=int(r/256.0)
            g=int(g/256.0)
            b=int(b/256.0)
        gimp.set_background(r,g,b)
        if img.layers[0].name== "Selector preview":
            pdb.gimp_edit_fill(img.layers[0],1)
            gimp.set_background(bgc)
            img.layers[0].opacity=100.0 
            pdb.gimp_displays_flush()

    def set_Lcolour(self, widget):
        h=self.col_left.get_color()
        r,g,b=hex_to_rgb(str(h))
        if len(str(h))==4:
            r=r*17
            g=g*17
            b=b*17
        elif len(str(h))>7:
            r=int(r/256.0)
            g=int(g/256.0)
            b=int(b/256.0)
        channel=self.ch.get_active()
        if channel==10:
            rc=float(self.hs_right.get_value())
            lc=hue(r,g,b)
            if lc>=rc:
                lc,rc=rc,lc
            self.hs_left.set_value(lc)
            self.hs_right.set_value(rc)
            self.set_left(self)
            self.set_right(self)
        if channel<10:
            self.cust.set_active(True)
            v=r
            if g>r: v=g
            if b>r: v=b
            rc=float(self.hs_right.get_value())
            lc=v
            if lc>=rc:
                lc,rc=rc,lc
            self.hs_left.set_value(lc)
            self.hs_right.set_value(rc)
            self.set_left(self)
            self.set_right(self)

    def set_Rcolour(self, widget):
        h=self.col_right.get_color()
        r,g,b=hex_to_rgb(str(h))
        if len(str(h))==4:
            r=r*17
            g=g*17
            b=b*17
        elif len(str(h))>7:
            r=int(r/256.0)
            g=int(g/256.0)
            b=int(b/256.0)
        channel=self.ch.get_active()
        if channel==10:
            lc=float(self.hs_left.get_value())
            rc=hue(r,g,b)
            if lc>=rc:
                lc,rc=rc,lc
            self.hs_left.set_value(lc)
            self.hs_right.set_value(rc)
            self.set_left(self)
            self.set_right(self)
        if channel<10:
            self.cust.set_active(True)
            v=r
            if g>r: v=g
            if b>r: v=b
            lc=float(self.hs_right.get_value())
            rc=v
            if lc>=rc:
                lc,rc=rc,lc
            self.hs_left.set_value(lc)
            self.hs_right.set_value(rc)
            self.set_left(self)
            self.set_right(self)

    def changed(self):
        self.test.set_sensitive(True), self.apply.set_sensitive(False)

class selector_plugin(gimpplugin.plugin):
    def start(self):
        gimp.main(self.init, self.quit, self.query, self._run)

    def init(self):
        pass

    def query(self):
        author = "Berthold Hinz"
        copyright = "Berthold Hinz"
        menu =  "<Image>/Select/Adagio Selector"
        year = "2010"
        description = "Creates range selection or layer mask for selected channel."
        help = "Creates range selection or layer mask for selected channel"
        parameters = [
                               (PDB_INT32, "run_mode", "Run mode"),
                               (PDB_IMAGE, "image", "Input image"),
                               (PDB_DRAWABLE, "drawable", "Input drawable"),
                             ]
        gimp.install_procedure("SelectorGUI",
                               description,
                               help,
                               author,
                               copyright,
                               year,
                               menu,
                               "*",
                               PLUGIN,  
                               parameters,
                               [])

    def SelectorGUI(self, run_mode, image, drawable):
        global img
        img=image
        global drw
        drw=drawable
        app = gui() 
        gtk.main()

if __name__ == '__main__':
    selector_plugin().start()
